package com.basic.eyflutter_core.subscribe;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.provider.Settings;

import com.basic.eyflutter_core.channel.OnDistributionSubscribe;
import com.basic.eyflutter_core.utils.UriFileUtils;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.JsonUtils;

import java.io.File;
import java.util.HashMap;

import io.flutter.plugin.common.MethodChannel;

public class InstallApkSubscribe extends OnDistributionSubscribe {

    private Context applicationContext;

    public InstallApkSubscribe(Context context) {
        this.applicationContext = context;
    }

    @Override
    public void onSubscribe(MethodChannel.Result result, HashMap<String, Object> arguments) {
        String path = ConvertUtils.toString(arguments.get("path"));
        File file = new File(path);
        installApk(result, file);
    }

    private void installApk(MethodChannel.Result result, File file) {
        boolean installAllowed;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            installAllowed = applicationContext.getPackageManager().canRequestPackageInstalls();
            if (!installAllowed) {
                String packageName = applicationContext.getPackageName();
                Uri uri = Uri.parse(String.format("package:%s", packageName));
                Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, uri);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                applicationContext.startActivity(intent);
                result.success(JsonUtils.toJson(new HashMap<String, Object>() {{
                    put("flag", "setting");
                }}));
                return;
            }
        }
        installApkFor7(result, file);
    }

    //安装apk，兼容7.0
    private void installApkFor7(MethodChannel.Result result, File file) {
        Intent intent = new Intent(Intent.ACTION_VIEW);
        // 由于没有在Activity环境下启动Activity,设置下面的标签   setFlags要放在addFlags之前
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        //版本在7.0以上是不能直接通过uri访问的
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            //参数1 上下文, 参数2 Provider主机地址和清单文件中保持一致   参数3 共享的文件
            Uri apkUri = UriFileUtils.getInstance().getUri(applicationContext, file);
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
        } else {
            intent.setDataAndType(Uri.fromFile(file),
                    "application/vnd.android.package-archive");
        }
        applicationContext.startActivity(intent);
        result.success(JsonUtils.toJson(new HashMap<String, Object>() {{
            put("flag", "done");
        }}));
    }
}
